@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2007 daniel g. siegel <dgsiegel@gmail.com>
- * Copyright (C) 2007 Jaap Haitsma <jaap@haitsma.org>
+ * Copyright (C) 2007,2008 daniel g. siegel <dgsiegel@gmail.com>
+ * Copyright (C) 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -22,7 +22,7 @@
 #define __CHEESE_FILE_UTIL_H__
 
 #define PHOTO_NAME_SUFFIX ".jpg"
-#define VIDEO_NAME_SUFFIX ".ogg"
+#define VIDEO_NAME_SUFFIX ".ogv"
 
 typedef enum
 {
