@@ -35,6 +35,10 @@
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
 
+#ifdef HILDON
+#include <hildon/hildon-program.h>
+#endif
+
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
 #include "cheese-fileutil.h"
@@ -43,7 +47,8 @@
 #include "cheese-window.h"
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
-
+#include "gedit-message-area.h"
+#include "cheese-no-camera.h"
 
 #define SHUTTER_SOUNDS 5
 
@@ -72,6 +77,7 @@ typedef struct
   GtkWidget *notebook_bar;
 
   GtkWidget *main_vbox;
+  GtkWidget *video_vbox;
 
   GtkWidget *effect_frame;
   GtkWidget *effect_chooser;
@@ -79,6 +85,8 @@ typedef struct
   GtkWidget *throbber;
   GtkWidget *countdown_frame;
   GtkWidget *countdown;
+  GtkWidget *message_area_frame;
+  GtkWidget *message_area;
 
   GtkWidget *button_effects;
   GtkWidget *button_photo;
@@ -97,16 +105,23 @@ typedef struct
   GtkWidget *screen;
   GtkWidget *take_picture;
 
-  GtkActionGroup *actions_main;
-  GtkActionGroup *actions_toggle;
+#ifdef HILDON
+   d
+   GtkWidget *main_hbox;
+   GtkWidget *subwindow;
+#endif      
+
+  GtkActionGroup *actions_account_photo;
+  GtkActionGroup *actions_countdown;
   GtkActionGroup *actions_effects;
   GtkActionGroup *actions_file;
-  GtkActionGroup *actions_photo;
-  GtkActionGroup *actions_video;
-  GtkActionGroup *actions_account_photo;
-  GtkActionGroup *actions_mail;
-  GtkActionGroup *actions_fspot;
   GtkActionGroup *actions_flickr;
+  GtkActionGroup *actions_fspot;
+  GtkActionGroup *actions_mail;
+  GtkActionGroup *actions_main;
+  GtkActionGroup *actions_photo;
+  GtkActionGroup *actions_toggle;
+  GtkActionGroup *actions_video;
 
   GtkUIManager *ui_manager;
 
@@ -210,15 +225,16 @@ cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
   g_free (cheese_window->rand);
   g_object_unref (cheese_window->webcam);
   g_object_unref (cheese_window->actions_main);
-  g_object_unref (cheese_window->actions_photo);
-  g_object_unref (cheese_window->actions_mail);
-  g_object_unref (cheese_window->actions_toggle);
+  g_object_unref (cheese_window->actions_account_photo);
+  g_object_unref (cheese_window->actions_countdown);
   g_object_unref (cheese_window->actions_effects);
   g_object_unref (cheese_window->actions_file);
-  g_object_unref (cheese_window->actions_video);
-  g_object_unref (cheese_window->actions_account_photo);
-  g_object_unref (cheese_window->actions_fspot);
   g_object_unref (cheese_window->actions_flickr);
+  g_object_unref (cheese_window->actions_fspot);
+  g_object_unref (cheese_window->actions_mail);
+  g_object_unref (cheese_window->actions_photo);
+  g_object_unref (cheese_window->actions_toggle);
+  g_object_unref (cheese_window->actions_video);
   g_object_unref (cheese_window->ui_manager);
   g_object_unref (cheese_window->gconf);
 
@@ -414,7 +430,7 @@ cheese_window_move_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_windo
   g_return_if_fail (filename);
 
   // return if the file does not exist, i.e. was already deleted
-  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS)) 
     return;
 
   file = g_file_new_for_path (filename);
@@ -425,6 +441,13 @@ cheese_window_move_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_windo
   g_list_free (files_list);
 }
 
+static void
+cheese_window_set_countdown (GtkWidget *widget, CheeseWindow *cheese_window)
+{
+  gboolean countdown = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (widget));
+  g_object_set (cheese_window->gconf, "gconf_prop_countdown", countdown, NULL);
+}
+
 static void
 cheese_window_cmd_set_about_me_photo (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -579,7 +602,7 @@ cheese_window_cmd_about (GtkAction *action, CheeseWindow *cheese_window)
   gtk_show_about_dialog (GTK_WINDOW (cheese_window->window),
                          "version", VERSION,
                          "copyright", "Copyright \xc2\xa9 2007, 2008\n daniel g. siegel <dgsiegel@gmail.com>",
-                         "comments", _("A cheesy program to take photos and videos from your webcam"),
+                         "comments", _("Take photos and videos with your webcam, with fun graphical effects"),
                          "authors", authors,
                          "translator-credits", translators,
                          "website", "http://www.gnome.org/projects/cheese",
@@ -763,7 +786,7 @@ cheese_window_photo_video_toggle_buttons_cb (GtkWidget *widget, CheeseWindow *ch
   {
     cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
 
-    str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
+    str = g_strconcat ("<b>", _("_Take a Photo"), "</b>", NULL);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
@@ -805,6 +828,30 @@ cheese_window_countdown_picture_cb (gpointer data)
   cheese_webcam_take_photo (cheese_window->webcam, cheese_window->photo_filename);
 }
 
+static void
+cheese_window_no_camera_message_area_response (GtkWidget *widget, gint response_id, GtkWidget *cheese_window)
+{
+  GError *error = NULL;
+  gboolean ret;
+
+  if (response_id == GTK_RESPONSE_HELP)
+  {
+    ret = g_app_info_launch_default_for_uri ("ghelp:cheese?faq", NULL, &error);
+
+    if (ret == FALSE) 
+    {
+      GtkWidget *d;
+      d = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window), 
+                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+                                  _("Unable to open help file for Cheese"));
+      gtk_dialog_run(GTK_DIALOG(d));
+      gtk_widget_destroy(d);
+      g_error_free(error);
+    }
+  }
+}
+
 static void
 cheese_window_stop_recording (CheeseWindow *cheese_window)
 {
@@ -814,7 +861,7 @@ cheese_window_stop_recording (CheeseWindow *cheese_window)
     gtk_widget_set_sensitive (cheese_window->button_effects, TRUE);
     gtk_widget_set_sensitive (cheese_window->button_photo, TRUE);
     gtk_widget_set_sensitive (cheese_window->button_video, TRUE);
-    gchar * str = g_strconcat ("<b>", _("_Start recording"), "</b>", NULL);
+    gchar * str = g_strconcat ("<b>", _("_Start Recording"), "</b>", NULL);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
@@ -876,7 +923,7 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
       gtk_widget_set_sensitive (cheese_window->button_effects, FALSE);
       gtk_widget_set_sensitive (cheese_window->button_photo, FALSE);
       gtk_widget_set_sensitive (cheese_window->button_video, FALSE);
-      str = g_strconcat ("<b>", _("_Stop recording"), "</b>", NULL);
+      str = g_strconcat ("<b>", _("_Stop Recording"), "</b>", NULL);
       gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
       g_free (str);
       gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
@@ -898,16 +945,20 @@ static const GtkActionEntry action_entries_main[] = {
   {"Cheese", NULL, N_("_Cheese")},
 
   {"Edit", NULL, N_("_Edit")},
-  {"RemoveAll", NULL, N_("Move all to Trash"), NULL, NULL, G_CALLBACK (cheese_window_move_all_media_to_trash)},
+  {"RemoveAll", NULL, N_("Move All to Trash"), NULL, NULL, G_CALLBACK (cheese_window_move_all_media_to_trash)},
 
   {"Help", NULL, N_("_Help")},
 
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (cheese_window_cmd_close)},
-  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Help on this application"), G_CALLBACK (cheese_window_cmd_help_contents)},
+  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Help on this Application"), G_CALLBACK (cheese_window_cmd_help_contents)},
   {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (cheese_window_cmd_about)},
 
 };
 
+static const GtkToggleActionEntry action_entries_countdown[] = {
+  {"Countdown", NULL, N_("Countdown"), NULL, NULL, G_CALLBACK (cheese_window_set_countdown), FALSE},
+};
+
 static const GtkToggleActionEntry action_entries_effects[] = {
   {"Effects", NULL, N_("_Effects"), NULL, NULL, G_CALLBACK (cheese_window_effect_button_pressed_cb), FALSE},
 };
@@ -917,7 +968,7 @@ static const GtkRadioActionEntry action_entries_toggle[] = {
   {"Video", NULL, N_("_Video"), NULL, NULL, 1},
 };
 
-static const GtkActionEntry action_entries_edit_file[] = {
+static const GtkActionEntry action_entries_file[] = {
   {"Open", GTK_STOCK_OPEN, N_("_Open"), "<control>O", NULL, G_CALLBACK (cheese_window_cmd_open)},
   {"SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<control>S", NULL, G_CALLBACK (cheese_window_cmd_save_as)},
   {"MoveToTrash", "user-trash", N_("Move to _Trash"), "Delete", NULL, G_CALLBACK (cheese_window_move_media_to_trash)},
@@ -1007,14 +1058,42 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
   return action_group;
 }
 
+static void
+cheese_window_set_message_area (CheeseWindow *cheese_window,
+                                GtkWidget *message_area)
+{
+  if (cheese_window->message_area == message_area)
+    return;
+
+  if (cheese_window->message_area != NULL)
+    gtk_widget_destroy (cheese_window->message_area);
+
+  cheese_window->message_area = message_area;
+
+  if (message_area == NULL)
+    return;
+
+  gtk_container_add (GTK_CONTAINER (cheese_window->message_area_frame), cheese_window->message_area);
+  gtk_widget_show (GTK_WIDGET(cheese_window->message_area));
+}
+
 static void
 cheese_window_create_window (CheeseWindow *cheese_window)
 {
-  GtkWidget *menubar;
   GError *error=NULL;
   char *path;
   GtkBuilder* builder;
 
+#ifdef HILDON
+  HildonProgram *program = hildon_program_get_instance();
+  GtkWidget *menu;
+  GtkWidget *menuitem;
+#else
+  GtkWidget *menubar;
+#endif
+
+  cheese_window->message_area = NULL;
+
   builder = gtk_builder_new ();
   gtk_builder_add_from_file (builder, PACKAGE_DATADIR"/cheese.ui", &error);
 
@@ -1034,6 +1113,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->label_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
   cheese_window->label_video        = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
   cheese_window->main_vbox          = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->video_vbox         = GTK_WIDGET (gtk_builder_get_object (builder, "video_vbox"));
   cheese_window->notebook           = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
   cheese_window->notebook_bar       = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
   cheese_window->screen             = GTK_WIDGET (gtk_builder_get_object (builder, "video_screen"));
@@ -1042,6 +1122,27 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->throbber_frame     = GTK_WIDGET (gtk_builder_get_object (builder, "throbber_frame"));
   cheese_window->countdown_frame    = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
   cheese_window->effect_frame       = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
+  cheese_window->message_area_frame = GTK_WIDGET (gtk_builder_get_object (builder, "message_area_frame"));
+
+#ifdef HILDON
+  /* Reparent widgets in case we use hildon. This saves us maintaining two
+     GtkBuilder ui files
+  */
+  cheese_window->window = hildon_window_new();
+  cheese_window->main_hbox = gtk_hbox_new(FALSE, 0);
+  hildon_program_add_window (program, HILDON_WINDOW(cheese_window->window));
+  gtk_container_add (GTK_CONTAINER(cheese_window->window), cheese_window->main_hbox);  
+  gtk_widget_ref(cheese_window->thumb_scrollwindow);
+  gtk_widget_ref(cheese_window->video_vbox);
+  gtk_container_remove(GTK_CONTAINER(cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
+  gtk_container_remove(GTK_CONTAINER(cheese_window->main_vbox),cheese_window->video_vbox);
+  gtk_box_pack_start (GTK_BOX(cheese_window->main_hbox), cheese_window->video_vbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX(cheese_window->main_hbox), GTK_WIDGET(cheese_window->thumb_scrollwindow), FALSE, FALSE, 0);
+  gtk_widget_destroy(cheese_window->main_vbox);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(cheese_window->thumb_scrollwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_widget_unref(cheese_window->thumb_scrollwindow);
+  gtk_widget_unref(cheese_window->video_vbox);
+#endif
 
   g_object_unref (builder);
 
@@ -1095,12 +1196,16 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                           G_N_ELEMENTS (action_entries_effects));
   cheese_window->actions_file = cheese_window_action_group_new (cheese_window, 
                                                                 "ActionsFile", 
-                                                                action_entries_edit_file, 
-                                                                G_N_ELEMENTS (action_entries_edit_file));
+                                                                action_entries_file, 
+                                                                G_N_ELEMENTS (action_entries_file));
   cheese_window->actions_photo = cheese_window_action_group_new (cheese_window, 
                                                                  "ActionsPhoto", 
                                                                  action_entries_photo, 
                                                                  G_N_ELEMENTS (action_entries_photo));
+  cheese_window->actions_countdown = cheese_window_toggle_action_group_new (cheese_window, 
+                                                                            "ActionsCountdown", 
+                                                                            action_entries_countdown, 
+                                                                            G_N_ELEMENTS (action_entries_countdown));
   cheese_window->actions_video = cheese_window_toggle_action_group_new (cheese_window, 
                                                                         "ActionsVideo", 
                                                                         action_entries_video, 
@@ -1143,8 +1248,33 @@ cheese_window_create_window (CheeseWindow *cheese_window)
     g_error_free (error);
   }
 
+  GtkAction *action = gtk_ui_manager_get_action (cheese_window->ui_manager, "/MainMenu/Cheese/CountdownToggle");
+  gboolean countdown;
+  g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+  if (countdown)
+  {
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+
+#ifdef HILDON
+  menu = gtk_menu_new();
+  menuitem = gtk_menu_item_new_with_label(_("Quit"));
+  g_signal_connect_swapped(menuitem,"activate",
+			   GTK_SIGNAL_FUNC(cheese_window_cmd_close),
+			   NULL);
+  gtk_menu_append(menu, menuitem);
+  
+  menuitem = gtk_menu_item_new_with_label(_("About"));
+  g_signal_connect_swapped(menuitem,"activate",
+			   GTK_SIGNAL_FUNC(cheese_window_cmd_about), 
+			   cheese_window->window);
+  gtk_menu_append(menu, menuitem);
+  
+  hildon_window_set_menu (HILDON_WINDOW(cheese_window->window),GTK_MENU(menu));
+#else
   menubar = gtk_ui_manager_get_widget (cheese_window->ui_manager, "/MainMenu");
   gtk_box_pack_start (GTK_BOX (cheese_window->main_vbox), menubar, FALSE, FALSE, 0);
+#endif
 
   cheese_window->thumb_view_popup_menu = gtk_ui_manager_get_widget (cheese_window->ui_manager, 
                                                                     "/ThumbnailPopup");
@@ -1176,6 +1306,7 @@ void
 setup_camera (CheeseWindow *cheese_window)
 {
   char *webcam_device = NULL;
+  GtkWidget *message_area;
 
   g_object_get (cheese_window->gconf, "gconf_prop_webcam", &webcam_device, NULL);
 
@@ -1196,6 +1327,17 @@ setup_camera (CheeseWindow *cheese_window)
   gdk_threads_enter ();
   gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
+  if (cheese_webcam_get_num_webcam_devices (cheese_window->webcam) == 0)
+  {
+    message_area = cheese_no_camera_message_area();
+
+    g_signal_connect (message_area,
+                      "response",
+                      G_CALLBACK (cheese_window_no_camera_message_area_response),
+                      cheese_window);
+
+    cheese_window_set_message_area (cheese_window, message_area);
+  }
   gdk_threads_leave ();
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), TRUE);
 }
@@ -1205,7 +1347,7 @@ cheese_window_init ()
 {
   CheeseWindow *cheese_window;
 
-  cheese_window = g_new0 (CheeseWindow, 1);
+  cheese_window = g_new (CheeseWindow, 1);
 
   cheese_window->gconf = cheese_gconf_new ();
   cheese_window->audio_play_counter = 0;
